# bb - bulk bin updater for sandbox
# (c) 2018 Benjamin Ingle
# Released under the GPL 2.1 License
# version 2 uses an ODBC connection to the DDMS table itself
#	and compares a hash of the table against a stored hash to
#	determine if an update is needed

import shutil, ConfigParser, hashlib, os, pyodbc, datetime, MySQLdb as mysql

def write_log(log_string):
#	""" Writes to logfile.  Param is the error string """
	date_today = datetime.datetime.strftime(datetime.datetime.now(), '%Y-%m-%d')
	time_now = datetime.datetime.strftime(datetime.datetime.now(), '%H%M%S')
	time_now = list(time_now)
	try:
		timestamp = time_now[0] + time_now[1] + ':' + time_now[2] + time_now[3] + ':' + time_now[4] + time_now[5]
		with open(file_params['logfile'], 'a') as logfile:
			logfile.write(str(date_today) + ' ' + str(timestamp) + ' : ' + str(log_string))
	except Exception as e:
		print str(e)

# read configuration
config = ConfigParser.ConfigParser()
config.readfp(open('bb2.cfg'))			# read config file
path_params = dict(config.items('paths'))
file_params = dict(config.items('files'))
db_params = dict(config.items('db'))

try:
	hashCon = mysql.connect(db_params['host'], db_params['mysql_user'], db_params['mysql_pass'], db_params['hash'])
	hashCur = hashCon.cursor()
except Exception as e:
	write_log("error connecting to MySQL server: " + str(e))

try: 	
	hashCur.execute("SELECT `hash` FROM `" + db_params['hashtable'] + "` WHERE table_name = '" + path_params['tablename'] + "' ORDER BY timestamp DESC LIMIT 1")
	mrHash = hashCur.fetchone()
except Exception as e:
	write_log("error retrieving hash from table: " + str(db_params['hashtable']))
# print 'mrHash: ' + str(mrHash[0])

hasher = hashlib.sha1()
with open(path_params['tablepath'] + path_params['tablename'], 'rb') as bulkFile:
	try:
		buffer = bulkFile.read(65536)
		while len(buffer) > 0:
			hasher.update(buffer)
			buffer = bulkFile.read(65536)
	except Exception as e:
		write_log("error hashing table: " + str(path_params['tablename']) + ": " + str(e))
# print(hasher.hexdigest())


if (mrHash[0] != hasher.hexdigest()):
	print "no match - updating table"
	# insert new hash into table
	hList = (path_params['tablename'], hasher.hexdigest())
	hashSql = "INSERT INTO " + db_params['hashtable'] + " (table_name, hash) VALUES (%s, %s)"
	try:
		hashCur.execute(hashSql, hList)
		hashCon.commit()
	except Exception as e:
		# print str(e)
		write_log("error executing hashCur in hash comparison: " + str(e))
		
	# create odbc connection to DDMS server
	try:
		dbfConn = pyodbc.connect('DSN=DDMS-IN')
		dbfCur = dbfConn.cursor()
	except Exception as e:
		write_log("error opening odbc connection to DDMS server: " + str(e))
		
	# create connection to MySQL server
	try:
		mariaCon = mysql.connect("localhost", db_params['mysql_user'], db_params['mysql_pass'], db_params['db'])
		mariaCur = mariaCon.cursor()
	except Exception as e:
		write_log("error connecting to MySQL server: " + str(e))

	try:
		mariaCur.execute("TRUNCATE TABLE " + db_params['bulkbin'])
	except Exception as e:
		write_log("error truncating table: " + str(db_params['bulkbin']) + ": " + str(e))

	try:
		dbfIter = dbfCur.execute("select * from i-bulk")
	except Exception as e:
		write_log("error getting data from i-bulk: " + str(e))

	valList = []
	k = 0
	for rec in dbfIter:
		try:
			valList.append((str(rec[0]).strip(), int(str(rec[1]).strip()), str(rec[2]).strip(), int(str(rec[3]).strip()), str(rec[4]).strip(), str(rec[5]).strip(), int(str(rec[6]).strip())))
			k += 1
		except Exception as e:
			write_log("error appending to list from query result on line: " + str(k) + " " + str(e))
		
	try:	
		sql = "INSERT INTO " + db_params['bulkbin'] + " (bulk_keys, table_order, item_key, loc, bin, bulk_unit, bulk_qty) VALUES (%s, %s, %s, %s, %s, %s, %s)"
		mariaCur.executemany(sql, valList)
	except Exception as e:
		write_log("error inserting values into table: " + str(db_params['bulkbin']) + ": " + str(e))
		
	hashCon.close()
	dbfConn.close()
	mariaCon.close()
else:
	print "match - no update needed"
	hashCon.close()